#!/usr/bin/python3
# ******************************************************************************
# Copyright (c) Huawei Technologies Co., Ltd. 2020-2021. All rights reserved.
# licensed under the Mulan PSL v2.
# You can use this software according to the terms and conditions of the Mulan PSL v2.
# You may obtain a copy of Mulan PSL v2 at:
#     http://license.coscl.org.cn/MulanPSL2
# THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, MERCHANTABILITY OR FIT FOR A PARTICULAR
# PURPOSE.
# See the Mulan PSL v2 for more details.
# ******************************************************************************/
"""
Parse config file, create database engines, obtains a specify database connection
"""
from packageship.application.common.exc import DatabaseConfigException
from packageship.application.database import engine
from packageship.libs.conf import configuration
from packageship.libs.log import LOGGER


class DatabaseSession(object):
    """
    Dynamic initialization of the database instance
    Raises: DatabaseConfigException, database is not support
    """
    __DATABASE_ENGINE_TYPE = ['elastic']

    def __init__(self, db_engine=None, host=None, port=None):
        self.db_engine = db_engine or "elastic"
        if self.db_engine not in self.__DATABASE_ENGINE_TYPE:
            LOGGER.error("DataBase %s is not support" % self.db_engine)
            raise DatabaseConfigException()
        self._host = host or configuration.DATABASE_HOST
        self._port = port or configuration.DATABASE_PORT
        self.session = None

    def connection(self):
        """
        Returns: specify database connection
        Raises: DatabaseConfigException, database class not found
        """
        self.session = engine.create_engine(db_engine=self.db_engine, host=self._host, port=self._port)
        if self.session is None:
            LOGGER.error("Failed to create database engine %s, please check database configuration" % self.db_engine)
            raise DatabaseConfigException()

        return self.session

    @property
    def client(self):
        """
        Get database client
        Returns: database client
        """
        if self.session is None:
            self.session = self.connection()

        return self.session.client
